@@ -15,54 +15,54 @@ use std::iter::FusedIterator;
 use std::marker::PhantomData;
 
 use crate::{
-    Mut, MutProxy, Proxied, SettableValue, View, ViewProxy,
-    __internal::{Private, RawRepeatedField},
-    __runtime::InnerRepeatedMut,
+    AsMut, AsView, IntoMut, IntoProxied, IntoView, Message, Mut, MutProxied, MutProxy, Proxied,
+    Proxy, View, ViewProxy,
+    __internal::runtime::{InnerRepeated, InnerRepeatedMut, RawRepeatedField},
+    __internal::{Private, SealedInternal},
 };
 
 /// Views the elements in a `repeated` field of `T`.
 #[repr(transparent)]
-pub struct RepeatedView<'msg, T: ?Sized> {
+pub struct RepeatedView<'msg, T> {
     // This does not need to carry an arena in upb, so it can be just the raw repeated field
     raw: RawRepeatedField,
     _phantom: PhantomData<&'msg T>,
 }
 
-impl<'msg, T: ?Sized> Copy for RepeatedView<'msg, T> {}
-impl<'msg, T: ?Sized> Clone for RepeatedView<'msg, T> {
+impl<'msg, T> Copy for RepeatedView<'msg, T> {}
+impl<'msg, T> Clone for RepeatedView<'msg, T> {
     fn clone(&self) -> Self {
         *self
     }
 }
 
-unsafe impl<'msg, T: ?Sized> Sync for RepeatedView<'msg, T> {}
-unsafe impl<'msg, T: ?Sized> Send for RepeatedView<'msg, T> {}
+unsafe impl<'msg, T> Sync for RepeatedView<'msg, T> {}
+unsafe impl<'msg, T> Send for RepeatedView<'msg, T> {}
 
-impl<'msg, T: ?Sized> Debug for RepeatedView<'msg, T> {
+impl<'msg, T> Debug for RepeatedView<'msg, T> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("RepeatedView").field("raw", &self.raw).finish()
     }
 }
 
 /// Mutates the elements in a `repeated` field of `T`.
-pub struct RepeatedMut<'msg, T: ?Sized> {
+pub struct RepeatedMut<'msg, T> {
     pub(crate) inner: InnerRepeatedMut<'msg>,
     _phantom: PhantomData<&'msg mut T>,
 }
 
-unsafe impl<'msg, T: ?Sized> Sync for RepeatedMut<'msg, T> {}
+unsafe impl<'msg, T> Sync for RepeatedMut<'msg, T> {}
 
-impl<'msg, T: ?Sized> Debug for RepeatedMut<'msg, T> {
+impl<'msg, T> Debug for RepeatedMut<'msg, T> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("RepeatedMut").field("raw", &self.inner.raw).finish()
     }
 }
 
-impl<'msg, T> RepeatedView<'msg, T>
-where
-    T: ProxiedInRepeated + ?Sized + 'msg,
-{
+#[doc(hidden)]
+impl<'msg, T> RepeatedView<'msg, T> {
     #[doc(hidden)]
+    #[inline]
     pub fn as_raw(&self, _private: Private) -> RawRepeatedField {
         self.raw
     }
@@ -70,16 +70,24 @@ where
     /// # Safety
     /// - `inner` must be valid to read from for `'msg`
     #[doc(hidden)]
+    #[inline]
     pub unsafe fn from_raw(_private: Private, raw: RawRepeatedField) -> Self {
         Self { raw, _phantom: PhantomData }
     }
+}
 
+impl<'msg, T> RepeatedView<'msg, T>
+where
+    T: ProxiedInRepeated + 'msg,
+{
     /// Gets the length of the repeated field.
+    #[inline]
     pub fn len(&self) -> usize {
         T::repeated_len(*self)
     }
 
     /// Returns true if the repeated field has no values.
+    #[inline]
     pub fn is_empty(&self) -> bool {
         self.len() == 0
     }
@@ -87,6 +95,7 @@ where
     /// Gets the value at `index`.
     ///
     /// Returns `None` if `index > len`.
+    #[inline]
     pub fn get(self, index: usize) -> Option<View<'msg, T>> {
         if index >= self.len() {
             return None;
@@ -99,6 +108,7 @@ where
     ///
     /// # Safety
     /// Undefined behavior if `index >= len`
+    #[inline]
     pub unsafe fn get_unchecked(self, index: usize) -> View<'msg, T> {
         // SAFETY: in-bounds as promised
         unsafe { T::repeated_get_unchecked(self, index) }
@@ -110,95 +120,123 @@ where
     }
 }
 
-impl<'msg, T> RepeatedMut<'msg, T>
-where
-    T: ProxiedInRepeated + ?Sized + 'msg,
-{
+#[doc(hidden)]
+impl<'msg, T> RepeatedMut<'msg, T> {
     /// # Safety
     /// - `inner` must be valid to read and write from for `'msg`
     /// - There must be no aliasing references or mutations on the same
     ///   underlying object.
     #[doc(hidden)]
+    #[inline]
     pub unsafe fn from_inner(_private: Private, inner: InnerRepeatedMut<'msg>) -> Self {
         Self { inner, _phantom: PhantomData }
     }
 
-    /// # Safety
-    /// - The return value must not be mutated through without synchronization.
-    #[allow(dead_code)]
-    pub(crate) unsafe fn into_inner(self) -> InnerRepeatedMut<'msg> {
-        self.inner
-    }
-
     #[doc(hidden)]
+    #[inline]
     pub fn as_raw(&mut self, _private: Private) -> RawRepeatedField {
         self.inner.raw
     }
+}
 
+impl<'msg, T> RepeatedMut<'msg, T>
+where
+    T: ProxiedInRepeated + 'msg,
+{
     /// Gets the length of the repeated field.
+    #[inline]
     pub fn len(&self) -> usize {
         self.as_view().len()
     }
 
     /// Returns true if the repeated field has no values.
+    #[inline]
     pub fn is_empty(&self) -> bool {
         self.len() == 0
     }
 
     /// Gets the value at `index`.
     ///
-    /// Returns `None` if `index > len`.
-    pub fn get(&self, index: usize) -> Option<View<T>> {
+    /// Returns `None` if `index >= len`.
+    #[inline]
+    pub fn get(&self, index: usize) -> Option<View<'_, T>> {
         self.as_view().get(index)
     }
 
+    /// Gets the value at `index`.
+    ///
+    /// Returns `None` if `index >= len`.
+    #[inline]
+    pub fn get_mut<'r>(&'r mut self, index: usize) -> Option<Mut<'msg, T>>
+    where
+        T: Message,
+        'r: 'msg,
+    {
+        if index >= self.len() {
+            return None;
+        }
+        // SAFETY: `index` has been checked to be in-bounds
+        Some(unsafe { self.get_mut_unchecked(index) })
+    }
+
     /// Gets the value at `index` without bounds-checking.
     ///
     /// # Safety
-    /// Undefined behavior if `index >= len`
-    pub unsafe fn get_unchecked(&self, index: usize) -> View<T> {
+    /// Undefined behavior if `index >= len`.
+    #[inline]
+    pub unsafe fn get_mut_unchecked<'r>(&'r mut self, index: usize) -> Mut<'msg, T>
+    where
+        T: Message,
+        'r: 'msg,
+    {
+        // SAFETY: in-bounds as promised
+        unsafe { T::repeated_get_mut_unchecked(self.as_mut(), index) }
+    }
+
+    /// Gets the value at `index` without bounds-checking.
+    ///
+    /// # Safety
+    /// Undefined behavior if `index >= len`.
+    #[inline]
+    pub unsafe fn get_unchecked(&self, index: usize) -> View<'_, T> {
         // SAFETY: in-bounds as promised
         unsafe { self.as_view().get_unchecked(index) }
     }
 
     /// Appends `val` to the end of the repeated field.
-    pub fn push(&mut self, val: View<T>) {
-        // TODO: b/320936046 - Use SettableValue instead of View for added ergonomics.
+    #[inline]
+    pub fn push(&mut self, val: impl IntoProxied<T>) {
         T::repeated_push(self.as_mut(), val);
     }
 
     /// Sets the value at `index` to the value `val`.
     ///
     /// # Panics
-    /// Panics if `index >= len`
-    pub fn set(&mut self, index: usize, val: View<T>) {
+    /// Panics if `index >= len`.
+    #[inline]
+    pub fn set(&mut self, index: usize, val: impl IntoProxied<T>) {
         let len = self.len();
         if index >= len {
             panic!("index {index} >= repeated len {len}");
         }
-        // TODO: b/320936046 - Use SettableValue instead of View for added ergonomics.
-        // SAFETY: `index` has been checked to be in-bounds.
         unsafe { self.set_unchecked(index, val) }
     }
 
     /// Sets the value at `index` to the value `val`.
     ///
     /// # Safety
-    /// Undefined behavior if `index >= len`
-    pub unsafe fn set_unchecked(&mut self, index: usize, val: View<T>) {
-        // TODO: b/320936046 - Use SettableValue instead of View for added ergonomics.
-        // SAFETY: `index` is in-bounds as promised by the caller.
+    /// Undefined behavior if `index >= len`.
+    #[inline]
+    pub unsafe fn set_unchecked(&mut self, index: usize, val: impl IntoProxied<T>) {
         unsafe { T::repeated_set_unchecked(self.as_mut(), index, val) }
     }
 
     /// Iterates over the values in the repeated field.
-    pub fn iter(&self) -> RepeatedIter<T> {
+    pub fn iter(&self) -> RepeatedIter<'_, T> {
         self.as_view().into_iter()
     }
 
     /// Copies from the `src` repeated field into this one.
-    ///
-    /// Also provided by [`MutProxy::set`].
     pub fn copy_from(&mut self, src: RepeatedView<'_, T>) {
         T::repeated_copy_from(src, self.as_mut())
     }
@@ -209,6 +247,53 @@ where
     }
 }
 
+impl<T> Repeated<T>
+where
+    T: ProxiedInRepeated,
+{
+    pub fn as_view(&self) -> View<'_, Repeated<T>> {
+        RepeatedView { raw: self.inner.raw(), _phantom: PhantomData }
+    }
+
+    #[doc(hidden)]
+    pub fn inner(&self, _private: Private) -> &InnerRepeated {
+        &self.inner
+    }
+}
+
+impl<'msg, T> IntoProxied<Repeated<T>> for RepeatedView<'msg, T>
+where
+    T: 'msg + ProxiedInRepeated,
+{
+    fn into_proxied(self, _private: Private) -> Repeated<T> {
+        let mut repeated: Repeated<T> = Repeated::new();
+        T::repeated_copy_from(self, repeated.as_mut());
+        repeated
+    }
+}
+
+impl<'msg, T> IntoProxied<Repeated<T>> for RepeatedMut<'msg, T>
+where
+    T: 'msg + ProxiedInRepeated,
+{
+    fn into_proxied(self, _private: Private) -> Repeated<T> {
+        IntoProxied::into_proxied(self.as_view(), _private)
+    }
+}
+
+impl<'msg, T, I, U> IntoProxied<Repeated<T>> for I
+where
+    I: Iterator<Item = U>,
+    T: 'msg + ProxiedInRepeated,
+    U: IntoProxied<T>,
+{
+    fn into_proxied(self, _private: Private) -> Repeated<T> {
+        let mut repeated: Repeated<T> = Repeated::new();
+        repeated.as_mut().extend(self);
+        repeated
+    }
+}
+
 /// Types that can appear in a `Repeated<T>`.
 ///
 /// This trait is implemented by generated code to communicate how the proxied
@@ -219,27 +304,23 @@ where
 /// # Safety
 /// - It must be sound to call `*_unchecked*(x)` with an `index` less than
 ///   `repeated_len(x)`.
-pub unsafe trait ProxiedInRepeated: Proxied {
+pub unsafe trait ProxiedInRepeated: Proxied + SealedInternal {
     /// Constructs a new owned `Repeated` field.
     #[doc(hidden)]
-    fn repeated_new(_private: Private) -> Repeated<Self> {
-        unimplemented!("not required")
-    }
+    fn repeated_new(_private: Private) -> Repeated<Self>;
 
     /// Frees the repeated field in-place, for use in `Drop`.
     ///
     /// # Safety
     /// - After `repeated_free`, no other methods on the input are safe to call.
     #[doc(hidden)]
-    unsafe fn repeated_free(_private: Private, _repeated: &mut Repeated<Self>) {
-        unimplemented!("not required")
-    }
+    unsafe fn repeated_free(_private: Private, _repeated: &mut Repeated<Self>);
 
     /// Gets the length of the repeated field.
     fn repeated_len(repeated: View<Repeated<Self>>) -> usize;
 
     /// Appends a new element to the end of the repeated field.
-    fn repeated_push(repeated: Mut<Repeated<Self>>, val: View<Self>);
+    fn repeated_push(repeated: Mut<Repeated<Self>>, val: impl IntoProxied<Self>);
 
     /// Clears the repeated field of elements.
     fn repeated_clear(repeated: Mut<Repeated<Self>>);
@@ -250,19 +331,38 @@ pub unsafe trait ProxiedInRepeated: Proxied {
 
     /// # Safety
     /// `index` must be less than `Self::repeated_len(repeated)`
-    unsafe fn repeated_set_unchecked(repeated: Mut<Repeated<Self>>, index: usize, val: View<Self>);
+    #[allow(unused_variables)]
+    unsafe fn repeated_get_mut_unchecked(repeated: Mut<Repeated<Self>>, index: usize) -> Mut<Self>
+    where
+        Self: Message,
+    {
+        panic!("repeated_get_mut_unchecked is only implemented for messages");
+    }
+
+    /// # Safety
+    /// `index` must be less than `Self::repeated_len(repeated)`
+    unsafe fn repeated_set_unchecked(
+        repeated: Mut<Repeated<Self>>,
+        index: usize,
+        val: impl IntoProxied<Self>,
+    );
 
     /// Copies the values in the `src` repeated field into `dest`.
     fn repeated_copy_from(src: View<Repeated<Self>>, dest: Mut<Repeated<Self>>);
+
+    /// Ensures that the repeated field has enough space allocated to insert at
+    /// least `additional` values without an allocation.
+    fn repeated_reserve(repeated: Mut<Repeated<Self>>, additional: usize);
 }
 
 /// An iterator over the values inside of a [`View<Repeated<T>>`](RepeatedView).
-pub struct RepeatedIter<'msg, T: ?Sized> {
+#[derive(Clone)]
+pub struct RepeatedIter<'msg, T> {
     view: RepeatedView<'msg, T>,
     current_index: usize,
 }
 
-impl<'msg, T: ?Sized> Debug for RepeatedIter<'msg, T> {
+impl<'msg, T> Debug for RepeatedIter<'msg, T> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("RepeatedIter")
             .field("view", &self.view)
@@ -271,78 +371,114 @@ impl<'msg, T: ?Sized> Debug for RepeatedIter<'msg, T> {
     }
 }
 
-/// An iterator over the mutators inside of a [`Mut<Repeated<T>>`](RepeatedMut).
-pub struct RepeatedIterMut<'msg, T: ?Sized> {
-    mutator: RepeatedMut<'msg, T>,
-    current_index: usize,
-}
-
-impl<'msg, T: ?Sized> Debug for RepeatedIterMut<'msg, T> {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        f.debug_struct("RepeatedIterMut")
-            .field("mutator", &self.mutator)
-            .field("current_index", &self.current_index)
-            .finish()
-    }
-}
-
 /// A `repeated` field of `T`, used as the owned target for `Proxied`.
 ///
 /// Users will generally write [`View<Repeated<T>>`](RepeatedView) or
 /// [`Mut<Repeated<T>>`](RepeatedMut) to access the repeated elements
-pub struct Repeated<T: ?Sized + ProxiedInRepeated> {
-    inner: InnerRepeatedMut<'static>,
+pub struct Repeated<T: ProxiedInRepeated> {
+    pub(crate) inner: InnerRepeated,
     _phantom: PhantomData<T>,
 }
 
-impl<T: ?Sized + ProxiedInRepeated> Repeated<T> {
-    #[allow(dead_code)]
-    pub(crate) fn new() -> Self {
+// SAFETY: `Repeated` is Sync because it does not implement interior mutability.
+unsafe impl<T: ProxiedInRepeated> Sync for Repeated<T> {}
+
+// SAFETY: `Repeated` is Send because it's not bound to a specific thread e.g.
+// it does not use thread-local data or similar.
+unsafe impl<T: ProxiedInRepeated> Send for Repeated<T> {}
+
+impl<T: ProxiedInRepeated> Repeated<T> {
+    pub fn new() -> Self {
         T::repeated_new(Private)
     }
-
-    pub(crate) unsafe fn from_inner(inner: InnerRepeatedMut<'static>) -> Self {
+    #[doc(hidden)]
+    pub fn from_inner(_private: Private, inner: InnerRepeated) -> Self {
         Self { inner, _phantom: PhantomData }
     }
 
-    #[allow(dead_code)]
-    pub(crate) fn inner(&mut self) -> InnerRepeatedMut<'static> {
-        self.inner
-    }
-
     pub(crate) fn as_mut(&mut self) -> RepeatedMut<'_, T> {
-        RepeatedMut { inner: self.inner, _phantom: PhantomData }
+        RepeatedMut { inner: self.inner.as_mut(), _phantom: PhantomData }
     }
 }
 
-impl<T: ?Sized + ProxiedInRepeated> Drop for Repeated<T> {
+impl<T: ProxiedInRepeated> Default for Repeated<T> {
+    fn default() -> Self {
+        Repeated::new()
+    }
+}
+
+impl<T: ProxiedInRepeated> Drop for Repeated<T> {
     fn drop(&mut self) {
         // SAFETY: only called once
         unsafe { T::repeated_free(Private, self) }
     }
 }
 
-// SAFETY: `Repeated` does not allow for shared mutability.
-unsafe impl<T: ProxiedInRepeated> Sync for Repeated<T> {}
-
 impl<T> Proxied for Repeated<T>
 where
-    T: ProxiedInRepeated + ?Sized,
+    T: ProxiedInRepeated,
 {
-    type View<'msg> = RepeatedView<'msg, T> where Repeated<T>: 'msg;
-    type Mut<'msg> = RepeatedMut<'msg, T> where Repeated<T>: 'msg;
+    type View<'msg>
+        = RepeatedView<'msg, T>
+    where
+        Repeated<T>: 'msg;
 }
 
-impl<'msg, T> ViewProxy<'msg> for RepeatedView<'msg, T>
+impl<T> SealedInternal for Repeated<T> where T: ProxiedInRepeated {}
+
+impl<T> AsView for Repeated<T>
 where
-    T: ProxiedInRepeated + ?Sized + 'msg,
+    T: ProxiedInRepeated,
+{
+    type Proxied = Self;
+
+    fn as_view(&self) -> RepeatedView<'_, T> {
+        self.as_view()
+    }
+}
+
+impl<T> MutProxied for Repeated<T>
+where
+    T: ProxiedInRepeated,
+{
+    type Mut<'msg>
+        = RepeatedMut<'msg, T>
+    where
+        Repeated<T>: 'msg;
+}
+
+impl<T> AsMut for Repeated<T>
+where
+    T: ProxiedInRepeated,
+{
+    type MutProxied = Self;
+
+    fn as_mut(&mut self) -> RepeatedMut<'_, T> {
+        self.as_mut()
+    }
+}
+
+impl<'msg, T> SealedInternal for RepeatedView<'msg, T> where T: ProxiedInRepeated + 'msg {}
+
+impl<'msg, T> Proxy<'msg> for RepeatedView<'msg, T> where T: ProxiedInRepeated + 'msg {}
+
+impl<'msg, T> AsView for RepeatedView<'msg, T>
+where
+    T: ProxiedInRepeated + 'msg,
 {
     type Proxied = Repeated<T>;
 
-    fn as_view(&self) -> View<'_, Self::Proxied> {
+    #[inline]
+    fn as_view(&self) -> View<'msg, Self::Proxied> {
         *self
     }
+}
 
+impl<'msg, T> IntoView<'msg> for RepeatedView<'msg, T>
+where
+    T: ProxiedInRepeated + 'msg,
+{
+    #[inline]
     fn into_view<'shorter>(self) -> View<'shorter, Self::Proxied>
     where
         'msg: 'shorter,
@@ -351,17 +487,30 @@ where
     }
 }
 
-impl<'msg, T> ViewProxy<'msg> for RepeatedMut<'msg, T>
+impl<'msg, T> ViewProxy<'msg> for RepeatedView<'msg, T> where T: ProxiedInRepeated + 'msg {}
+
+impl<'msg, T> SealedInternal for RepeatedMut<'msg, T> where T: ProxiedInRepeated + 'msg {}
+
+impl<'msg, T> Proxy<'msg> for RepeatedMut<'msg, T> where T: ProxiedInRepeated + 'msg {}
+
+impl<'msg, T> AsView for RepeatedMut<'msg, T>
 where
-    T: ProxiedInRepeated + ?Sized + 'msg,
+    T: ProxiedInRepeated + 'msg,
 {
     type Proxied = Repeated<T>;
 
-    fn as_view(&self) -> View<'_, Self::Proxied> {
+    #[inline]
+    fn as_view(&self) -> RepeatedView<'_, T> {
         RepeatedView { raw: self.inner.raw, _phantom: PhantomData }
     }
+}
 
-    fn into_view<'shorter>(self) -> View<'shorter, Self::Proxied>
+impl<'msg, T> IntoView<'msg> for RepeatedMut<'msg, T>
+where
+    T: ProxiedInRepeated + 'msg,
+{
+    #[inline]
+    fn into_view<'shorter>(self) -> RepeatedView<'shorter, T>
     where
         'msg: 'shorter,
     {
@@ -369,15 +518,24 @@ where
     }
 }
 
-impl<'msg, T> MutProxy<'msg> for RepeatedMut<'msg, T>
+impl<'msg, T> AsMut for RepeatedMut<'msg, T>
 where
-    T: ProxiedInRepeated + ?Sized + 'msg,
+    T: ProxiedInRepeated + 'msg,
 {
-    fn as_mut(&mut self) -> Mut<'_, Self::Proxied> {
+    type MutProxied = Repeated<T>;
+
+    #[inline]
+    fn as_mut(&mut self) -> RepeatedMut<'_, T> {
         RepeatedMut { inner: self.inner, _phantom: PhantomData }
     }
+}
 
-    fn into_mut<'shorter>(self) -> Mut<'shorter, Self::Proxied>
+impl<'msg, T> IntoMut<'msg> for RepeatedMut<'msg, T>
+where
+    T: ProxiedInRepeated + 'msg,
+{
+    #[inline]
+    fn into_mut<'shorter>(self) -> RepeatedMut<'shorter, T>
     where
         'msg: 'shorter,
     {
@@ -385,25 +543,15 @@ where
     }
 }
 
-impl<'msg, T> SettableValue<Repeated<T>> for RepeatedView<'msg, T>
-where
-    T: ProxiedInRepeated + ?Sized + 'msg,
-{
-    fn set_on<'b>(self, _private: Private, mutator: Mut<'b, Repeated<T>>)
-    where
-        Repeated<T>: 'b,
-    {
-        T::repeated_copy_from(self, mutator)
-    }
-}
+impl<'msg, T> MutProxy<'msg> for RepeatedMut<'msg, T> where T: ProxiedInRepeated + 'msg {}
 
-// TODO: impl ExactSizeIterator
 impl<'msg, T> iter::Iterator for RepeatedIter<'msg, T>
 where
-    T: ProxiedInRepeated + ?Sized + 'msg,
+    T: ProxiedInRepeated + 'msg,
 {
     type Item = View<'msg, T>;
 
+    #[inline]
     fn next(&mut self) -> Option<Self::Item> {
         let val = self.view.get(self.current_index);
         if val.is_some() {
@@ -411,19 +559,25 @@ where
         }
         val
     }
-}
 
-impl<'msg, T: ?Sized + ProxiedInRepeated> ExactSizeIterator for RepeatedIter<'msg, T> {
-    fn len(&self) -> usize {
-        self.view.len()
+    fn size_hint(&self) -> (usize, Option<usize>) {
+        let len = self.len();
+        (len, Some(len))
     }
 }
 
-impl<'msg, T: ?Sized + ProxiedInRepeated> FusedIterator for RepeatedIter<'msg, T> {}
+impl<'msg, T: ProxiedInRepeated> ExactSizeIterator for RepeatedIter<'msg, T> {
+    fn len(&self) -> usize {
+        self.view.len() - self.current_index
+    }
+}
+
+// TODO: impl DoubleEndedIterator
+impl<'msg, T: ProxiedInRepeated> FusedIterator for RepeatedIter<'msg, T> {}
 
 impl<'msg, T> iter::IntoIterator for RepeatedView<'msg, T>
 where
-    T: ProxiedInRepeated + ?Sized + 'msg,
+    T: ProxiedInRepeated + 'msg,
 {
     type Item = View<'msg, T>;
     type IntoIter = RepeatedIter<'msg, T>;
@@ -435,7 +589,7 @@ where
 
 impl<'msg, T> iter::IntoIterator for &'_ RepeatedView<'msg, T>
 where
-    T: ProxiedInRepeated + ?Sized + 'msg,
+    T: ProxiedInRepeated + 'msg,
 {
     type Item = View<'msg, T>;
     type IntoIter = RepeatedIter<'msg, T>;
@@ -447,7 +601,7 @@ where
 
 impl<'borrow, T> iter::IntoIterator for &'borrow RepeatedMut<'_, T>
 where
-    T: ProxiedInRepeated + ?Sized + 'borrow,
+    T: ProxiedInRepeated + 'borrow,
 {
     type Item = View<'borrow, T>;
     type IntoIter = RepeatedIter<'borrow, T>;
@@ -457,12 +611,26 @@ where
     }
 }
 
+impl<'msg, 'view, T, ViewT> Extend<ViewT> for RepeatedMut<'msg, T>
+where
+    T: ProxiedInRepeated + 'view,
+    ViewT: IntoProxied<T>,
+{
+    fn extend<I: IntoIterator<Item = ViewT>>(&mut self, iter: I) {
+        let iter = iter.into_iter();
+        T::repeated_reserve(self.as_mut(), iter.size_hint().0);
+        for item in iter {
+            self.push(item);
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
     use googletest::prelude::*;
 
-    #[test]
+    #[gtest]
     fn test_primitive_repeated() {
         macro_rules! primitive_repeated_tests {
             ($($t:ty => [$($vals:expr),* $(,)?]),* $(,)?) => {
@@ -484,8 +652,7 @@ mod tests {
                     assert_that!(r.len(), eq(expected_len));
 
                 )*
-                assert_that!(
-                    r.iter().collect::<Vec<$t>>(), elements_are![$(eq($vals)),*]);
+                assert_that!(r, elements_are![$(eq($vals)),*]);
                 r.set(0, <$t as Default>::default());
                 assert_that!(r.get(0).expect("elem 0"), eq(<$t as Default>::default()));
 
@@ -503,4 +670,33 @@ mod tests {
             bool => [false, true, true, false],
         );
     }
+
+    #[gtest]
+    fn test_repeated_extend() {
+        let mut r = Repeated::<i32>::new();
+
+        r.as_mut().extend([0; 0]);
+        assert_that!(r.as_mut().len(), eq(0));
+
+        r.as_mut().extend([0, 1]);
+        assert_that!(r.as_mut(), elements_are![eq(0), eq(1)]);
+        let mut x = Repeated::<i32>::new();
+        x.as_mut().extend([2, 3]);
+
+        r.as_mut().extend(&x.as_mut());
+
+        assert_that!(r.as_mut(), elements_are![eq(0), eq(1), eq(2), eq(3)]);
+    }
+
+    #[gtest]
+    fn test_repeated_iter() {
+        let r: Repeated<i32> = [0, 1, 2, 3].into_iter().into_proxied(Private);
+        assert_that!(r.as_view(), elements_are![eq(0), eq(1), eq(2), eq(3)]);
+
+        let mut iter = r.as_view().into_iter();
+        assert_that!(iter.next(), eq(Some(0)));
+        let mut clone = iter.clone();
+        assert_that!(clone.next(), eq(Some(1)));
+        assert_that!(iter.next(), eq(Some(1)));
+    }
 }
