@@ -41,6 +41,7 @@
 #include "google/protobuf/message_lite.h"
 #include "google/protobuf/port.h"
 #include "google/protobuf/repeated_field.h"
+#include "google/protobuf/repeated_ptr_field.h"
 #include "google/protobuf/test_util.h"
 #include "google/protobuf/unittest.pb.h"
 #include "google/protobuf/unittest_arena.pb.h"
@@ -531,7 +532,8 @@ class DispatcherTestProto : public Message {
   using InternalArenaConstructable_ = void;
   using DestructorSkippable_ = void;
   // For the test below to construct.
-  explicit DispatcherTestProto(absl::in_place_t) : Message(nullptr, nullptr) {}
+  explicit constexpr DispatcherTestProto(absl::in_place_t)
+      : Message(static_cast<internal::ClassData*>(nullptr)) {}
   explicit DispatcherTestProto(Arena*) : Message(nullptr, nullptr) {
     ABSL_LOG(FATAL);
   }
@@ -543,6 +545,8 @@ class DispatcherTestProto : public Message {
     ABSL_LOG(FATAL);
   }
 };
+DispatcherTestProto dispatcher_test_proto_instance(absl::in_place);
+
 // We use a specialization to inject behavior for the test.
 // This test is very intrusive and will have to be fixed if we change the
 // implementation of CreateMessage.
@@ -550,18 +554,18 @@ absl::string_view hook_called;
 template <>
 void* Arena::DefaultConstruct<DispatcherTestProto>(Arena*) {
   hook_called = "default";
-  return nullptr;
+  return &dispatcher_test_proto_instance;
 }
 template <>
 void* Arena::CopyConstruct<DispatcherTestProto>(Arena*, const void*) {
   hook_called = "copy";
-  return nullptr;
+  return &dispatcher_test_proto_instance;
 }
 template <>
 DispatcherTestProto* Arena::CreateArenaCompatible<DispatcherTestProto, int>(
     Arena*, int&&) {
   hook_called = "fallback";
-  return nullptr;
+  return &dispatcher_test_proto_instance;
 }
 
 TEST(ArenaTest, CreateArenaConstructable) {
@@ -604,7 +608,7 @@ TEST(ArenaTest, CreateMessageDispatchesToSpecialFunctions) {
   Arena::Create<DispatcherTestProto>(nullptr);
   EXPECT_EQ(hook_called, "default");
 
-  DispatcherTestProto ref(absl::in_place);
+  DispatcherTestProto& ref = dispatcher_test_proto_instance;
   const DispatcherTestProto& cref = ref;
 
   hook_called = "";
@@ -1375,16 +1379,16 @@ TEST(ArenaTest, ArenaOneofReflection) {
 void TestSwapRepeatedField(Arena* arena1, Arena* arena2) {
   // Test "safe" (copying) semantics for direct Swap() on RepeatedPtrField
   // between arenas.
-  RepeatedPtrField<TestAllTypes> field1(arena1);
-  RepeatedPtrField<TestAllTypes> field2(arena2);
+  auto* field1 = Arena::Create<RepeatedPtrField<TestAllTypes>>(arena1);
+  auto* field2 = Arena::Create<RepeatedPtrField<TestAllTypes>>(arena2);
   for (int i = 0; i < 10; i++) {
     TestAllTypes* t = Arena::Create<TestAllTypes>(arena1);
     t->set_optional_string("field1");
     t->set_optional_int32(i);
     if (arena1 != nullptr) {
-      field1.UnsafeArenaAddAllocated(t);
+      field1->UnsafeArenaAddAllocated(t);
     } else {
-      field1.AddAllocated(t);
+      field1->AddAllocated(t);
     }
   }
   for (int i = 0; i < 5; i++) {
@@ -1392,22 +1396,29 @@ void TestSwapRepeatedField(Arena* arena1, Arena* arena2) {
     t->set_optional_string("field2");
     t->set_optional_int32(i);
     if (arena2 != nullptr) {
-      field2.UnsafeArenaAddAllocated(t);
+      field2->UnsafeArenaAddAllocated(t);
     } else {
-      field2.AddAllocated(t);
+      field2->AddAllocated(t);
     }
   }
-  field1.Swap(&field2);
-  EXPECT_EQ(5, field1.size());
-  EXPECT_EQ(10, field2.size());
-  EXPECT_TRUE(std::string("field1") == field2.Get(0).optional_string());
-  EXPECT_TRUE(std::string("field2") == field1.Get(0).optional_string());
+  field1->Swap(field2);
+  EXPECT_EQ(5, field1->size());
+  EXPECT_EQ(10, field2->size());
+  EXPECT_TRUE(std::string("field1") == field2->Get(0).optional_string());
+  EXPECT_TRUE(std::string("field2") == field1->Get(0).optional_string());
   // Ensure that fields retained their original order:
-  for (int i = 0; i < field1.size(); i++) {
-    EXPECT_EQ(i, field1.Get(i).optional_int32());
+  for (int i = 0; i < field1->size(); i++) {
+    EXPECT_EQ(i, field1->Get(i).optional_int32());
   }
-  for (int i = 0; i < field2.size(); i++) {
-    EXPECT_EQ(i, field2.Get(i).optional_int32());
+  for (int i = 0; i < field2->size(); i++) {
+    EXPECT_EQ(i, field2->Get(i).optional_int32());
+  }
+
+  if (arena1 == nullptr) {
+    delete field1;
+  }
+  if (arena2 == nullptr) {
+    delete field2;
   }
 }
 
@@ -1455,25 +1466,26 @@ TEST(ArenaTest, RepeatedFieldOnArena) {
     // newly allocated memory is approximately the size of the cleanups for the
     // repeated messages.
     RepeatedField<int32_t> repeated_int32(&arena);
-    RepeatedPtrField<TestAllTypes> repeated_message(&arena);
+    auto* repeated_message =
+        Arena::Create<RepeatedPtrField<TestAllTypes>>(&arena);
     for (int i = 0; i < 100; i++) {
       repeated_int32.Add(42);
-      repeated_message.Add()->set_optional_int32(42);
-      EXPECT_EQ(&arena, repeated_message.Get(0).GetArena());
-      const TestAllTypes* msg_in_repeated_field = &repeated_message.Get(0);
-      TestAllTypes* msg = repeated_message.UnsafeArenaReleaseLast();
+      repeated_message->Add()->set_optional_int32(42);
+      EXPECT_EQ(&arena, repeated_message->Get(0).GetArena());
+      const TestAllTypes* msg_in_repeated_field = &repeated_message->Get(0);
+      TestAllTypes* msg = repeated_message->UnsafeArenaReleaseLast();
       EXPECT_EQ(msg_in_repeated_field, msg);
     }
 
     // UnsafeArenaExtractSubrange (i) should not leak and (ii) should return
     // on-arena pointers.
     for (int i = 0; i < 10; i++) {
-      repeated_message.Add()->set_optional_int32(42);
+      repeated_message->Add()->set_optional_int32(42);
     }
     TestAllTypes* extracted_messages[5];
-    repeated_message.UnsafeArenaExtractSubrange(0, 5, extracted_messages);
-    EXPECT_EQ(&arena, repeated_message.Get(0).GetArena());
-    EXPECT_EQ(5, repeated_message.size());
+    repeated_message->UnsafeArenaExtractSubrange(0, 5, extracted_messages);
+    EXPECT_EQ(&arena, repeated_message->Get(0).GetArena());
+    EXPECT_EQ(5, repeated_message->size());
     // Upper bound of the size of the cleanups of new repeated messages.
     const size_t upperbound_cleanup_size =
         2 * 110 * sizeof(internal::cleanup::CleanupNode);
@@ -1483,14 +1495,15 @@ TEST(ArenaTest, RepeatedFieldOnArena) {
 
   // Now test ExtractSubrange's copying semantics.
   {
-    RepeatedPtrField<TestAllTypes> repeated_message(&arena);
+    auto* repeated_message =
+        Arena::Create<RepeatedPtrField<TestAllTypes>>(&arena);
     for (int i = 0; i < 100; i++) {
-      repeated_message.Add()->set_optional_int32(42);
+      repeated_message->Add()->set_optional_int32(42);
     }
 
     TestAllTypes* extracted_messages[5];
     // ExtractSubrange should copy to the heap.
-    repeated_message.ExtractSubrange(0, 5, extracted_messages);
+    repeated_message->ExtractSubrange(0, 5, extracted_messages);
     EXPECT_EQ(nullptr, extracted_messages[0]->GetArena());
     // We need to free the heap-allocated messages to prevent a leak.
     for (int i = 0; i < 5; i++) {
